﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;

namespace Dlubal.Examples.AreaLoads
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void btnSetAreaLoad_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        if (data.GetMemberCount() > 0)
        {
          // Gets interface to loads.
          ILoads loads = model.GetLoads();

          LoadCase lc;

          // Gets load case No. or creates new load case if no one exists.
          if (loads.GetLoadCaseCount() > 0)
          {
            lc = loads.GetLoadCase(0, ItemAt.AtIndex).GetData();
          }
          else
          {
            lc = new LoadCase();
            lc.Loading.No = 1;
            lc.ActionCategory = ActionCategoryType.Permanent;
            lc.Description = "My Load Case";
            //lc.Analysis.LoadingFactor = 1.0;

            // Creates new load case.
            loads.PrepareModification();
            try
            {
              loads.SetLoadCase(lc);
            }
            catch (Exception ex)
            {
              MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
              loads.FinishModification();
            }
          }

          // Area Load
          AreaLoad load = new AreaLoad();
          load.LoadCaseNo = lc.Loading.No;
          load.Direction = LoadDirectionType.GlobalZType;
          load.MemberLoadDirection = DirectionType.LocalDirection;
          load.Application = AreaLoadApplicationType.FullyClosedPlaneType;
          load.DistributionType = AreaLoadDistributionType.ConstantType;
          load.Magnitude1 = 10000.0;
          load.CornerNodes = "1,9,4";

          // Generates area load
          loads.GenerateAreaLoad(ref load);
        }
        else
        {
          MessageBox.Show("No member found in the model.");
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}