﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;
using Dlubal.STEELM;

namespace Dlubal.Examples.Modules.SteelMembers
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void btnRun_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to module RF-STEEL Members.
        IstModule module = model.GetModule("STEEL_M") as IstModule;

        // Sets new case No.1 to module RF-STEEL Members.
        IstCase moduleCase = module.moSetCase(1, "My new case");

        // Sets all members.
        moduleCase.stSetMemberList("all");

        // Sets load case No.1.
        ILOAD[] loads = new ILOAD[1];
        loads[0].type = ILOAD_TYPE.ILOAD_CASE;
        loads[0].No = 1;
        moduleCase.stSetLoadsArr(1, loads);

        // Calculates results.
        moduleCase.stResultsCalculate();

        // Gets module results.
        IstResults results = moduleCase.stGetResults();

        // Gets stresses by members.
        int count = results.stGetStressMCount();
        ST_RESULTS_STRESSES[] stresses = new ST_RESULTS_STRESSES[count];
        count = results.stGetStressMArr(count, stresses);
        for (int i = 0; i < count; i++)
        {
          ListViewItem item = new ListViewItem();
          item.Text = stresses[i].iMemberNo.ToString();
          item.SubItems.Add(stresses[i].xLocation.ToString("0.000"));
          item.SubItems.Add(stresses[i].iPoint.ToString());
          item.SubItems.Add(stresses[i].sLCNo);
          item.SubItems.Add(stresses[i].eStressType.ToString());
          item.SubItems.Add(stresses[i].values[(int)ST_STRESS_VALUE.SV_EXISTING].ToString("0.000"));
          item.SubItems.Add(stresses[i].values[(int)ST_STRESS_VALUE.SV_ALLOWABLE].ToString("0.000"));
          item.SubItems.Add(stresses[i].values[(int)ST_STRESS_VALUE.SV_RATIO].ToString("0.000"));

          listViewResults.Items.Add(item);
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
