﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;

namespace Dlubal.Examples.SelectObjects
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void buttonSelect_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      ModelObjectType type;
      if (radioNodes.Checked)
        type = ModelObjectType.NodeObject;
      else if (radioLines.Checked)
        type = ModelObjectType.LineObject;
      else if (radioMembers.Checked)
        type = ModelObjectType.MemberObject;
      else if (radioSurfaces.Checked)
        type = ModelObjectType.SurfaceObject;
      else
      {
        MessageBox.Show("Invalid type of objects");
        return;
      }

      if (string.IsNullOrEmpty(textNumbers.Text))
      {
        MessageBox.Show("You have to write objects numbers");
        textNumbers.Focus();
        return;
      }

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        data.SelectObjects(type, textNumbers.Text);

        data = null;
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
