﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;

namespace Dlubal.Examples.SurfaceLoads
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void btnSetSurfaceLoad_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        if (data.GetSurfaceCount() > 0)
        {
          // Gets interface to loads.
          ILoads loads = model.GetLoads();

          ILoadCase lc = null;

          // Gets load case No. or creates new load case if no one exists.
          if (loads.GetLoadCaseCount() > 0)
          {
            lc = loads.GetLoadCase(0, ItemAt.AtIndex);
          }
          else
          {
            LoadCase lcData = new LoadCase();
            lcData.Loading.No = 1;
            lcData.ActionCategory = ActionCategoryType.Permanent;
            lcData.Description = "My Load Case";
            //lcData.Analysis.LoadingFactor = 1.0;

            // Creates new load case.
            loads.PrepareModification();
            try
            {
              lc = loads.SetLoadCase(lcData);
            }
            catch (Exception ex)
            {
              MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
              loads.FinishModification();
            }
          }

          // Surface Load
          SurfaceLoad load = new SurfaceLoad();
          load.No = 1;
          load.SurfaceList = "1"; // on surface No.1
          load.Direction = LoadDirectionType.GlobalZType;
          load.Distribution = LoadDistributionType.LinearType;
          load.Node1No = 1;
          load.Node2No = 2;
          load.Node3No = 3;
          load.Magnitude1 = 10000.0;
          load.Magnitude2 = 20000.0;
          load.Magnitude3 = 30000.0;

          // Creates new surface load.
          lc.PrepareModification();
          try
          {
            lc.SetSurfaceLoad(load);
          }
          catch (Exception ex)
          {
            MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
          }
          finally
          {
            lc.FinishModification();
          }
        }
        else
        {
          MessageBox.Show("No surface found in the model.");
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
