﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RSTAB8;
using Dlubal.TimberPro;

namespace TIMBERProExample
{
  public partial class MainForm : Form
  {
    public MainForm()
    {
      InitializeComponent();
    }

    private void btnStart_Click(object sender, System.EventArgs e)
    {
      /*
       * You have to add reference to following files:
       * 
       *   C:\Windows\assembly\GAC_MSIL\Dlubal.RSTAB8\2.3.0.0__fea8abdc389fb11c\Dlubal.RSTAB8.dll
       *   C:\Windows\Microsoft.NET\assembly\GAC_MSIL\Dlubal.TimberPro\v4.0_1.0.0.0__0b06332e21b255cd\Dlubal.TimberPro.dll
       * 
       * For both these files you have to set option "Embed Interop Types" to False in reference properties window.
       * 
      */
      IModel model = null;

      try
      {
        // Connects to the running RSTAB and it's first opened model.
        model = Marshal.GetActiveObject("RSTAB8.Model") as IModel;

        // Creates a new RSTAB instance and opens an empty model.
        // model = new Dlubal.RSTAB8.Structure();

        // Checks the license and prepare for COM for using.
        model.GetApplication().LockLicense();

        // Gets interface for working with the module TIMBER Pro.
        IModule module = model.GetModule("TimberPro") as IModule;

        // If the module contains a module case, gets it's interface, otherwise creates a new module case.
        IModuleCase moduleCase = null;
        if (module.GetCaseCount() > 0)
        {
          moduleCase = module.GetCase(0, Dlubal.TimberPro.ItemAt.AtIndex) as IModuleCase;
        }
        else
        {
          moduleCase = module.SetCase(1, "test") as IModuleCase;

          // Set all member for analysis.
          moduleCase.SetMemberList("all");

          // If there is no loading selected, add LC1.
          UltimateLimitState[] uls = new UltimateLimitState[1];
          uls[0].No = 1;
          uls[0].Type = Dlubal.TimberPro.LoadingType.LoadCaseType;
          uls[0].DesignSituation = Dlubal.TimberPro.DesignSituationType.PersistentAndTransientType;
          moduleCase.SetULSLoads(uls);

          // Sets loading duration
          LoadDurationDetail detail = new LoadDurationDetail();
          detail.LoadID.No = 1;
          detail.LoadID.Type = Dlubal.TimberPro.LoadingType.LoadCaseType;
          detail.LoadDurationClassForModuleUse = LoadDurationClass.LoadDurationPermanent;
          moduleCase.SetLoadDurationDetail(detail);
        }

        // Gets all module case materials.
        Dlubal.TimberPro.Material[] material = moduleCase.GetMaterials();

        // Gets all load used in the module case.
        LoadDurationDetail[] loads = moduleCase.GetLoadDurationDetails();
        
        // Gets service class details.
        ServiceClassDetails serviceClass = moduleCase.GetServiceClassDetails();

        // Gets fire resistances.
        FireResistance[] fireResistances = moduleCase.GetMembersFireResistances();
        // Gets fire resistance class.
        FireResistanceOptions options = moduleCase.GetFireResistanceOptions();

        // If there are no results, calculate them.
        if (!moduleCase.HasResults())
        {
          CalcError[] errors = moduleCase.Calculate();
          if (errors.Length > 0)
            MessageBox.Show(errors[0].Description);
        }

        // If there are results, read them.
        if (moduleCase.HasResults())
        {
          // Gets interface for working with module case results.
          ITimberProResults results = moduleCase.GetResults();

          // Gets internal forces
          ResultsInternalForces[] internalForces = results.GetInternalForcesByMember();
        }

        // We need to release all used interfaces when we don't need them anymore.
        moduleCase = null;
        module = null;
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source);
      }
      finally
      {
        // Releases RS-COM license and update the model.
        if (model != null)
          model.GetApplication().UnlockLicense();
      }

      // Empty garbage collector and release all cached interfaces.
      System.GC.Collect();
      System.GC.WaitForPendingFinalizers();
    }
  }
}
