﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RSTAB8;

namespace ResultsExample
{
  public partial class MainForm : Form
  {
    public MainForm()
    {
      InitializeComponent();
    }

    private void btnReadMIF_Click(object sender, EventArgs e)
    {
      /*
       * You have to add reference to following files:
       * 
       *   C:\Windows\assembly\GAC_MSIL\Dlubal.RSTAB8\2.3.0.0__fea8abdc389fb11c\Dlubal.RSTAB8.dll
       *   C:\Windows\Microsoft.NET\assembly\GAC_MSIL\Dlubal.TimberPro\v4.0_1.0.0.0__0b06332e21b255cd\Dlubal.TimberPro.dll
       * 
       * For both these files you have to set option "Embed Interop Types" to False in reference properties window.
       * 
      */
      IModel model = null;

      try
      {
        // Connects to the running RSTAB and it's first opened model.
        model = Marshal.GetActiveObject("RSTAB8.Model") as IModel;

        // Checks the license and prepare for COM for using.
        model.GetApplication().LockLicense();

        // Gets interface for working with results.
        ICalculation calculation = model.GetCalculation();

        // Calculate all results.
        calculation.CalculateAll();

        // Gets result of load case No.1.
        IResults results = calculation.GetResults(LoadingType.LoadCaseType, 1);

        // Reads results
        MemberForces[] resultsMIF = results.GetMembersInternalForces(false);

        foreach (MemberForces mif in resultsMIF)
        {
          double n = mif.Forces.X;
          double vy = mif.Forces.Y;
          double vz = mif.Forces.Z;
          double mt = mif.Moments.X;
          double my = mif.Moments.Y;
          double mz = mif.Moments.Z;
        }

        results = null;
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source);
      }
      finally
      {
        // Releases RS-COM license and update the model.
        if (model != null)
          model.GetApplication().UnlockLicense();
      }

      // Empty garbage collector and release all cached interfaces.
      System.GC.Collect();
      System.GC.WaitForPendingFinalizers();
    }
  }
}
