﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RSTAB8;

namespace Dlubal.Examples.SetMembers
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void btnSetMembers_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RSTAB application.
        app = Marshal.GetActiveObject("RSTAB8.Application") as IApplication;
        // Locks RSTAB licence
        app.LockLicense();

        // Gets interface to active RSTAB model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        // Allocates array of 4 nodes and sets theirs parameters.
        Node[] nodes = new Node[4];
        nodes[0].No = 1;
        nodes[0].X = 0;
        nodes[0].Y = 0;
        nodes[0].Z = 0;

        nodes[1].No = 2;
        nodes[1].X = 1;
        nodes[1].Y = 0;
        nodes[1].Z = 0;

        nodes[2].No = 3;
        nodes[2].X = 1;
        nodes[2].Y = 1;
        nodes[2].Z = 0;

        nodes[3].No = 4;
        nodes[3].X = 0;
        nodes[3].Y = 1;
        nodes[3].Z = 0;

        // Allocates cross-section variable and sets it's parameters.
        CrossSection crsc = new CrossSection();
        crsc.No = 1;
        crsc.MaterialNo = 1;
        crsc.Description = "IPE 100";

        // Allocates array of 4 members and sets theirs parameters.
        Member[] members = new Member[4];
        members[0].No = 1;
        members[0].StartCrossSectionNo = 1;
        members[0].EndCrossSectionNo = 1;
        members[0].Type = MemberType.Beam;
        members[0].StartNodeNo = 1;
        members[0].EndNodeNo = 2;

        members[1].No = 2;
        members[1].StartCrossSectionNo = 1;
        members[1].EndCrossSectionNo = 1;
        members[1].Type = MemberType.Beam;
        members[1].StartNodeNo = 2;
        members[1].EndNodeNo = 3;

        members[2].No = 3;
        members[2].StartCrossSectionNo = 1;
        members[2].EndCrossSectionNo = 1;
        members[2].Type = MemberType.Beam;
        members[2].StartNodeNo = 3;
        members[2].EndNodeNo = 4;

        members[3].No = 4;
        members[3].StartCrossSectionNo = 1;
        members[3].EndCrossSectionNo = 1;
        members[3].Type = MemberType.Beam;
        members[3].StartNodeNo = 4;
        members[3].EndNodeNo = 1;

        // Sets all objects to model data.
        data.PrepareModification();
        try
        {
          data.SetNodes(nodes);
          data.SetCrossSection(crsc);
          data.SetMembers(members);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          data.FinishModification();
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RSTAB model.
        if (model != null)
        {
          model = null;
        }

        // Unlocks licence and releases interface to RSTAB application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces and objects.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
