// RS COM ExampleDlg.cpp : implementation file
#include "StdAfx.h"
#include "RS-COM.h"
#include "RS-COM_Dlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CRSCOMDlg dialog
CRSCOMDlg::CRSCOMDlg(CWnd* pParent /*=NULL*/)	: CDialog(CRSCOMDlg::IDD, pParent)
{
}

BEGIN_MESSAGE_MAP(CRSCOMDlg, CDialog)
  ON_BN_CLICKED(IDOK, &CRSCOMDlg::OnBnClickedOk)
END_MESSAGE_MAP()

inline CString GetComErrorMessage(const _com_error &e)
{
  CString sDescription = (TCHAR*)e.Description();
  CString sErrorMessage = (TCHAR*)e.ErrorMessage();
  CString sSource = (TCHAR*)e.Source();

  sSource.TrimRight();

  CString sNewError;
  if(!sSource.IsEmpty())
    sNewError = sSource;
  if(!sErrorMessage.IsEmpty())
    sNewError = sNewError + ": " + sErrorMessage;
  if(!sDescription.IsEmpty()) 
    sNewError = sNewError + " (" + sDescription + ")";

  return sNewError;
}

void CRSCOMDlg::OnBnClickedOk()
{
  RSTAB6::IrsStructurePtr pIStructure;

  try
  {
    pIStructure.GetActiveObject(__uuidof(RSTAB6::Structure)); // get running instance of RSTAB6
    if (pIStructure == NULL)
      pIStructure.CreateInstance(__uuidof(RSTAB6::Structure));  // create new instance of RSTAB

    pIStructure->rsGetApplication()->rsLockLicence();

    // prepare RSTAB for manipulation
    pIStructure->rsGetStructuralData()->rsPrepareModification();

    // create array of nodes (because usually there are many of nodes and array is faster)
    RSTAB6::RS_NODE *pArrNode = new RSTAB6::RS_NODE[2];

    RSTAB6::RS_NODE oNodeBeg;
    ZeroMemory(&oNodeBeg, sizeof(RSTAB6::RS_NODE));

    oNodeBeg.iNo = 1;
    oNodeBeg.x = oNodeBeg.y = oNodeBeg.z = 0;

    pArrNode[0] = oNodeBeg;

    RSTAB6::RS_NODE oNodeEnd;
    ZeroMemory(&oNodeEnd, sizeof(RSTAB6::RS_NODE));

    oNodeEnd.iNo = 2;
    oNodeEnd.x = 1;
    oNodeEnd.y = oNodeEnd.z = 0;

    pArrNode[1] = oNodeEnd;

    // fill up nodes
    pIStructure->rsGetStructuralData()->rsSetNodeArr(2, pArrNode);

    // create a material
    RSTAB6::RS_MATERIAL oMaterial;
    ZeroMemory(&oMaterial, sizeof(RSTAB6::RS_MATERIAL));

    oMaterial.iNo = 1;
    oMaterial.strDescription = SysAllocString(L"S235");

    pIStructure->rsGetStructuralData()->rsSetMaterial(&oMaterial);

    // create a cross section
    RSTAB6::RS_CROSS_SECTION oCrSc;
    ZeroMemory(&oCrSc, sizeof(RSTAB6::RS_CROSS_SECTION));

    oCrSc.iNo = 1;

    oCrSc.iMaterialNo = 1;
    oCrSc.strDescription = SysAllocString(L"IPE 200");

    pIStructure->rsGetStructuralData()->rsSetCrossSection(&oCrSc);

    // finally create a member
    RSTAB6::RS_MEMBER oMember;
    ZeroMemory(&oMember, sizeof(RSTAB6::RS_MEMBER));

    oMember.iNo = 1;

    oMember.iStartCrossSectionNo = oCrSc.iNo;
    oMember.iEndCrossSectionNo = oCrSc.iNo;

    oMember.iStartNodeNo = oNodeBeg.iNo;
    oMember.iEndNodeNo = oNodeEnd.iNo;

    pIStructure->rsGetStructuralData()->rsSetMember(&oMember);

    // finish manipulation
    pIStructure->rsGetStructuralData()->rsFinishModification();
  }
  catch(_com_error e)
  {
    CString sMeesage = GetComErrorMessage(e);
    AfxMessageBox(sMeesage, MB_ICONERROR);
  }

  // clean up
  if (pIStructure != NULL)
  {
    pIStructure->rsGetApplication()->rsShowApplication();
    pIStructure->rsGetApplication()->rsUnlockLicence();

    pIStructure.Release(); // disconnet from the structure
  }
}
